
// read_vmd.cc
// This code is used for reading velocity meta data
//

#include <stdio.h>
#include <stdlib.h>
#include <strings.h>

static void Usage() {
   printf("Usage: read_vmd <velocity_metadata_par>\n");
   printf("  Options : \n");
   printf("    -line : line position\n");
   printf("   -pixel : pixel position\n");
   printf("       -x : easting coordinate\n");
   printf("       -y : northing coordinate\n");
   printf("     -out : output filename\n");
}


int main(int argc, char **argv) 
{
  int k, line = -1, pixel = -1, number_of_columns, number_of_rows;
  double x = 0, y = 0, ul_easting, ul_northing;
  double  easting_spacing, northing_spacing;
  char metadata_par_filename[1024], metadata_filename[1024];
  char conflation_filename[1024], index_filename[1024];
  char string[1024], out_filename[1024];
  char s1[1024], s2[1024];
  FILE *meta_fp;
  FILE *conf_fp;
  float inputs[8];
  unsigned char nr_measurement;
  int floatsize;
  int charsize;
  int intsize;
  int byte_position;
  unsigned char method;
  float speed_error;
  unsigned short index;
  double easting;
  double northing;
  FILE *out_fp, *fp;
  char **methods;
  int *indexes;
  int effective_indexes;
  char **index_strings;
  FILE *ind_fp; 
  int index_printed = 0;
  int nr_vcps = 0;
  int kk;
  int number_of_datatakes = 0;

  int meta_point_bytes;

  if(argc == 1) {
    Usage();  return 0;
  }
  else if(argc > 11) {
    printf("Too Many command parameters !!!!!! \n");
    return 0;
  }

  strcpy(metadata_par_filename, argv[1]);

  for(k=1; k<argc; k++) {
    if(strcmp(argv[k], "-h") == 0 || strcmp(argv[k], "-help") == 0) {
        Usage(); return 0; }
    if(strcmp(argv[k], "-line") == 0 && k+1 < argc) 
        line = atoi(argv[k+1]);
    if(strcmp(argv[k], "-pixel") == 0 && k+1 < argc) 
        pixel = atoi(argv[k+1]);
    if(strcmp(argv[k], "-x") == 0 && k+1 < argc) x = atof(argv[k+1]);
    if(strcmp(argv[k], "-y") == 0 && k+1 < argc) y = atof(argv[k+1]);
    if(strcmp(argv[k], "-out") == 0 && k+1 < argc) 
       strcpy(out_filename, argv[k+1]);
    }


  fp = fopen(metadata_par_filename, "r");
  if(fp == NULL) {
    printf("The metadata parameter file %s does not exist !!! \n",
          metadata_par_filename);
    return 0;
  }


  while(fgets(string, 1024, fp)) {
    sscanf(string, "%s %s ", s1, s2);
    if(strcmp(s1, "metadata_filename:") == 0) 
        strcpy(metadata_filename, s2);
    if(strcmp(s1, "metadata_par_filename:") == 0)
        strcpy(metadata_par_filename, s2);
    if(strcmp(s1, "conflation_filename:") == 0)
        strcpy(conflation_filename, s2);
    if(strcmp(s1, "index_filename:") == 0) 
        strcpy(index_filename, s2);
    if(strcmp(s1, "number_of_columns:") == 0) 
        number_of_columns = atoi(s2);
    if(strcmp(s1, "number_of_rows:") == 0) 
        number_of_rows = atoi(s2);
    if(strcmp(s1, "upper_left_easting:") == 0) 
        ul_easting = atof(s2);
    if(strcmp(s1, "upper_left_northing:") == 0) 
        ul_northing = atof(s2);
    if(strcmp(s1, "easting_spacing:") == 0) 
        easting_spacing = atof(s2);
    if(strcmp(s1, "northing_spacing:") == 0) 
        northing_spacing = atof(s2);
     }
  fclose(fp);


  if(line == -1 || pixel == -1) {
    pixel  = (x - ul_easting) / easting_spacing;
    line = (ul_northing - y) / northing_spacing;
    }
  if(line < 0 || line >= number_of_rows || pixel < 0 || pixel >= number_of_columns) {
     printf( "Location specified is out of range !!! \n" );
     printf( "Please try again !!!!!!!!!!!!!!!!!!!!! \n" );
  }

  meta_fp = fopen(metadata_filename, "r");
  conf_fp = fopen(conflation_filename, "r");
  floatsize = sizeof(float);
  charsize = sizeof(unsigned char);
  intsize = sizeof(int);
  meta_point_bytes = (floatsize * 8 + intsize)*(line*number_of_columns + pixel);

  fseek(meta_fp, meta_point_bytes, SEEK_SET);
  fread(inputs, floatsize, 8, meta_fp);
  fread(&byte_position, intsize, 1, meta_fp);

  easting = ul_easting + pixel*easting_spacing;
  northing = ul_northing - line*northing_spacing;

  out_fp = fopen(out_filename, "w");
  if(out_fp) {
    fprintf(out_fp,"Velocity Meta Data information : \n");
    fprintf(out_fp,"    line :            %d \n", line);
    fprintf(out_fp,"    pixel :           %d \n", pixel);
    fprintf(out_fp,"    easting  :        %f \n", easting);
    fprintf(out_fp,"    northing :        %f \n", northing);
    fprintf(out_fp,"    elevation :       %f \n", inputs[0]);
    fprintf(out_fp,"    slope_x :         %f \n", inputs[1]);
    fprintf(out_fp,"    slope_y :         %f \n", inputs[2]);
    fprintf(out_fp,"    Vx :              %f \n",inputs[3] );
    fprintf(out_fp,"    Vy :              %f \n",inputs[4] );
    fprintf(out_fp,"    Vz :              %f \n",inputs[5] );
    fprintf(out_fp,"    speed_error :     %f \n",inputs[6] );
    fprintf(out_fp,"    direction_error : %f \n",inputs[7] );
  }
  else {
    printf("Velocity Meta Data information : \n");
    printf("    line :            %d \n", line);
    printf("    pixel :           %d \n", pixel);
    printf("    easting  :        %f \n", easting);
    printf("    northing :        %f \n", northing);
    printf("    elevation :       %f \n", inputs[0]);
    printf("    slope_x :         %f \n", inputs[1]);
    printf("    slope_y :         %f \n", inputs[2]);
    printf("    Vx :              %f \n",inputs[3] );
    printf("    Vy :              %f \n",inputs[4] );
    printf("    Vz :              %f \n",inputs[5] );
    printf("    speed_error :     %f \n",inputs[6] );
    printf("    direction_error : %f \n",inputs[7] );
  }

  methods = (char **)calloc(4, sizeof(char *));
  for(k=0; k<4; k++) methods[k] = (char *)calloc(16, 1);
  strcpy(methods[0], "Offsets only");
  strcpy(methods[1], "Regular");
  strcpy(methods[2], "Azimuth");
  strcpy(methods[3], "Range");

  fseek(conf_fp, byte_position, SEEK_SET);
  fread(&nr_measurement, charsize, 1, conf_fp);
  if((int)nr_measurement > 0) {

  indexes = (int *)calloc((int)nr_measurement, sizeof(int));
  effective_indexes = 0;

  for(k=0; k<(int)nr_measurement; k++) {
      fread(&method, sizeof(unsigned char), 1, conf_fp);
      fread(&speed_error, sizeof(float), 1, conf_fp);
      fread(&index, sizeof(unsigned short), 1, conf_fp);
      indexes[k] = (int)index;
      if(out_fp) {
         fprintf(out_fp, 
           " conflation index: %2d   method: %12s speed_error: %12.3f   datatake index: %d \n",
              k, methods[(int)(method)], speed_error, index);
      }
      else {
        printf(
          "conflation index: %2d method: %12s speed_error: %12.3f datatake index: %d \n", 
               k, methods[(int)(method)], speed_error, index);
      }
      if(k > 0) {
         for(kk=0; kk < k; kk++) {
            if(index == indexes[kk]) indexes[k] = -1;
          }
      }
      if(indexes[k] >= 0) effective_indexes ++;
    }
 index_strings = (char **)calloc(effective_indexes, sizeof(char *));
 effective_indexes = 0;
 for(k=0; k<(int)nr_measurement; k++) {
    if(indexes[k] < 0) continue;
    index_strings[effective_indexes] = (char *)calloc(8, 1);
    sprintf(index_strings[effective_indexes], "%d", indexes[k]);
    strcat(index_strings[effective_indexes], ":");
    effective_indexes ++;
    }

  ind_fp = fopen(index_filename, "r");
  number_of_datatakes = 0;
  fscanf(ind_fp, "%s %d", string, &number_of_datatakes);

  index_printed = 0;
  nr_vcps = 0;
  while(fgets(string, 1023, ind_fp)) {
      sscanf(string, "%s", s1);
      for(k=0; k<effective_indexes; k++) {
         if(strcmp(index_strings[k], "done") != 0) {
           if(strcmp(index_strings[k], s1) == 0) {
            if(out_fp) {
              fprintf(out_fp,"%s", string);
             }
          else {
            printf("%s", string);
            }
          strcpy(index_strings[k], "done");
          index_printed ++;
          sscanf(string, "%s %s %s %s %s %s %s %s %d", s1, s1, s1, s1,
          s1, s1, s1, s1, &nr_vcps);
          if(nr_vcps > 0) {
             for(kk=0; kk<nr_vcps; kk++) {
               if(fgets(string, 1023, ind_fp)) {
                 if(out_fp) {
                    fprintf(out_fp, "%s", string);
                  } else {
                    printf("%s", string);
                  }
               }
             }
           }
         }
       }
     }
     if(index_printed >= effective_indexes) break;
  }

    free(indexes);
    for(k=0; k<effective_indexes; k++) free(index_strings[k]);
    free(index_strings);

  }

  for(k=0; k<4; k++) free(methods[k]);
  free(methods);

  fclose(meta_fp);
  fclose(conf_fp);

}


